/*
This file exists so that you will understand the demo. I suggest
reading it.

The purpose of this demo is to alleviate a problem the so commonly plagues
beginners. You see, many times, beginners, when making an equipment
system for their game, do it in completely the wrong way, design-wise.There may
not be an absolute right way to do anything, but there are certainly wrong ways.
One of the WRONG ways to make an equipment system would be something like like this:

mob/var/equipped=0//BAD

obj/Inventory/Equipment/verb

    Equip()

        set src in usr //Only usable if src is in usr.contents

        if(!usr.equipped)//BAD
            usr.equipped=1//BAD

    Unequip()

        set src in usr

        if(usr.equipped)//BAD
            usr.equipped=0//BAD


Looks fine, right? Wrong.

You see, what you have here is a boolean mob.equipped variable. A "boolean"
variable is a variable that is either 1 or 0; TRUE or FALSE. Boolean variables,
are, of course, not inherently bad. However, they shouldn't be used for something
like this. Of course, you're probably wondering why.

You see, in the type of system shown above, you have nothing to link the mob
with his equipment. A 1 or 0 simply doesn't tell you anything. Sure, you know that
SOMETHING is equipped, but you don't know quite what that something is.

The other issue here is that using a boolean mob.equipped is simply not robust,
If mob.equipped were accidently to get set to 0 when it really should be 1, then
what happens? Everything goes screwy. The player's statistics could go wrong, possibly
resulting in a negative value, depending on the rest of your program!

The key to a well-engineered program is not that everything works perfectly, but
rather that if one component of the program fails, the rest of the program won't
be affected.

That said, this demo is meant to teach a better way:

Instead of being a 1 or a 0, the mob's equipment variables are equal to the actual
weapon that the mob has equipped. Now, we have something to connect the player with
his equipment!

If you've already looked through the demo, you may be wondering about my use of
the built-in associative list, mob.vars. For those who don't know, atom.vars is an
associative list that contains each and every one of an atom's variables. They are referenced
by their variable names and they are associated with their values. If you want to learn about
associative lists, Lummox JR has an excellent BYONDscape article on them which I strongly
recommend reading.


Here's why I'm using mob.vars:

--------------

By defining things like mob/var/obj/Inventory/Equipment/Weapon/weapon to store
the equipped weapon in, it enables the developer to directly modify the weapon:


src.weapon.strength+=10


However, without the use of an associative list and an identifier string, the code
has to go through istype() checks to figure out whether, for example,
obj/Inventory/Equipment/E should be saved to mob.weapon or mob.shield:


mob/Player
    var/Inventory/Equipment
        Weapon/weapon
        Shield/shield

    proc/Equip(obj/Inventory/Equipment/E)
        if(istype(E,/obj/Inventory/Equipment/Weapon))
            src.weapon=E
        if(istype(E,/obj/Inventory/Equipment/Shield))
            src.shield=E


The problem there is that if the equipped weapon is only accessable from the
associative list, then the developer would have to do things like this to modify it:


var/obj/Inventory/Equipment/Weapon/W=src.Equipment["weapon"]
W.strength+=10


Using mob.vars brings out the best of both worlds because although you can still
modify the equipment using things like:


src.weapon.strength+=10


While preventing all of the if(istype()) checks:


obj/Inventory/Equipment
    var/identifier

    Weapon
        identifier="weapon"
    Shield
        identifier="shield"

mob/Player
    var/Inventory/Equipment
        Weapon/weapon
        Shield/shield

    proc/Equip(obj/Inventory/Equipment/E)
        src.vars["[E.identifier]"]=E

--------------

Anyway, if you have any questions, then post them on the Equipment Demo forum, and
I'll be glad to answer them.


-- Wizkidd0123
*/